

#include <setupapi.h>
#include <tchar.h>	// Note UNICODE may not work for Win98??

#define DEVINTF_INLINE inline

//////////////////////////////////////////////////////////////////////////////
// class CDeviceInterfaceClass
//
class CDeviceInterfaceClass
{
public:
	CDeviceInterfaceClass(GUID* pClassGuid, PDWORD status);
	~CDeviceInterfaceClass(void);
	GUID* GetGuid(void)      { return &m_Guid; }
	HDEVINFO GetHandle(void) { return m_hInfo; }

protected:
	HDEVINFO		m_hInfo;
	GUID			m_Guid;
};

//////////////////////////////////////////////////////////////////////////////
// class CDeviceInterface
//
class CDeviceInterface
{
public:
	DEVINTF_INLINE CDeviceInterface( CDeviceInterfaceClass* pClassObject, DWORD Index, PDWORD Error );
	DEVINTF_INLINE ~CDeviceInterface(void);
	DEVINTF_INLINE TCHAR* DevicePath(void);

protected:
	CDeviceInterfaceClass*				m_Class;
	SP_DEVICE_INTERFACE_DATA			m_Data;
	PSP_INTERFACE_DEVICE_DETAIL_DATA	m_Detail;
};


//////////////////////////////////////////////////////////////////////////////
// CDeviceInterfaceClass constructor
//
DEVINTF_INLINE CDeviceInterfaceClass::CDeviceInterfaceClass(
	GUID* pClassGuid, 
	PDWORD status
	) 
{
	DWORD flags = DIGCF_DEVICEINTERFACE | DIGCF_PRESENT ;
	m_hInfo = INVALID_HANDLE_VALUE;
	ZeroMemory(&m_Guid,sizeof(GUID));

	try
	{
		*status = ERROR_INVALID_PARAMETER;
		m_Guid = *pClassGuid;
		m_hInfo = SetupDiGetClassDevs(pClassGuid, NULL, NULL, flags);

		if ( m_hInfo == INVALID_HANDLE_VALUE )
			*status = GetLastError();
		else
			*status = ERROR_SUCCESS;

	}
	catch (...)
	{
		m_hInfo = INVALID_HANDLE_VALUE;
	}
}

//////////////////////////////////////////////////////////////////////////////
// CDeviceInterfaceClass destructor
//
DEVINTF_INLINE CDeviceInterfaceClass::~CDeviceInterfaceClass(void)
{
	if ( m_hInfo != INVALID_HANDLE_VALUE )
		SetupDiDestroyDeviceInfoList(m_hInfo);

	m_hInfo = INVALID_HANDLE_VALUE;
}

//////////////////////////////////////////////////////////////////////////////
// CDeviceInterface constructor
//
DEVINTF_INLINE CDeviceInterface::CDeviceInterface(
	CDeviceInterfaceClass* pClassObject, 
	DWORD Index,
	PDWORD Error
	)
{
	m_Class = pClassObject;

	BOOL status;
	DWORD ReqLen;
	
	m_Detail = NULL;
	m_Data.cbSize = sizeof SP_DEVICE_INTERFACE_DATA;

	try
	{
		*Error = ERROR_INVALID_PARAMETER;

		status = SetupDiEnumDeviceInterfaces(
			m_Class->GetHandle(), 
			NULL, 
			m_Class->GetGuid(), 
			Index, 
			&m_Data
			);

		if ( !status )
		{
			*Error = GetLastError();
			return;
		}					  

		SetupDiGetInterfaceDeviceDetail (
			m_Class->GetHandle(),
			&m_Data,
			NULL,
			0,
			&ReqLen,
			NULL 
			);

		*Error = GetLastError();

		if ( *Error != ERROR_INSUFFICIENT_BUFFER )
			return;

		m_Detail = PSP_INTERFACE_DEVICE_DETAIL_DATA(new char[ReqLen]);

		if ( !m_Detail )
		{
			*Error = ERROR_NOT_ENOUGH_MEMORY;
			return;
		}

		m_Detail->cbSize = sizeof SP_INTERFACE_DEVICE_DETAIL_DATA;

		status = SetupDiGetInterfaceDeviceDetail (
			m_Class->GetHandle(),
			&m_Data,
			m_Detail,
			ReqLen,
			&ReqLen,
			NULL 
			);

		if ( !status )
		{
			*Error = GetLastError();
			delete m_Detail;
			m_Detail = NULL;
			return;
		}
	
		*Error = ERROR_SUCCESS;
	}
	catch (...)
	{
	}
}

//////////////////////////////////////////////////////////////////////////////
// CDeviceInterface destructor
//
DEVINTF_INLINE CDeviceInterface::~CDeviceInterface(void)
{
	if (m_Detail)
	{
		delete m_Detail;
		m_Detail = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
// CDeviceInterface::DevicePath
//
DEVINTF_INLINE TCHAR* CDeviceInterface::DevicePath(void)
{
	try
	{
		if ( m_Detail)
			return m_Detail->DevicePath;
		else
			return NULL;
	}
	catch (...)
	{
		return NULL;
	}

}


/*
GUID m_ClassGuid = AecUsbTCDevice_CLASS_GUID;


CAecMfcUSBDlg::~CAecMfcUSBDlg()
{
    if (!CloseHandle(m_hAecDevice))           // Close the Device "file".
    {
        //printf("Failed to close device.\n");
		MessageBox("Unable to Close the Aec USB device.",
			NULL, MB_ICONSTOP | MB_OK);
    }
	//KillTimer(m_Dlg_Timer);

}


	DWORD	Error;


	m_hAecDevice = INVALID_HANDLE_VALUE;
	m_hAecDevice = OpenByInterface( &m_ClassGuid, 0, &Error);
    if (m_hAecDevice == INVALID_HANDLE_VALUE)        // Was the device opened?
    {
        //printf("Unable to open the device.\n");
		//int MessageBox(HWND hWnd, LPCTSTR lpText, LPCTSTR lpCaption, UINT uType); 
		//iMB = MessageBox(hwnd, "Unable to open the Aec USB device.",
		//	NULL, //default is "Error" if null "Caption-Unable to open the device.\n", 
		//	MB_ICONSTOP | MB_OK); 
		MessageBox("Unable to open the Aec USB device.",
			NULL, //default is "Error" if null "Caption-Unable to open the device.\n", 
			MB_ICONSTOP | MB_OK);

        exit(1);
    }





//-------------------------------------------------------
BOOL CAecMfcUSBDlg::DoReadWriteAction(ULONG act, ULONG port, PULONG pdata)
{
	BOOL status = false;
	AECPCI_ACTION_ITEM		ai;			// *PAECPCI_ACTION_ITEM;
	DWORD nOutput = 0;

	ai.action = (AECPCI_ACTION)act;
	ai.port = port;
	ai.value = *pdata;

	//Nonzero indicates success. 
	//Zero indicates failure. 
	//To get extended error information, call GetLastError. 

	//status = TRUE;		//TEMP debug
	//*pdata = 0x13;		//TEMP debug

	if (m_hAecDevice != INVALID_HANDLE_VALUE)        // Was the device opened?
	{
		if (status = DeviceIoControl(		
                m_hAecDevice,			//1 Handle to device
                IOCTL_AECPCI_PROCESS_BUFFER,//2 IO Control code -- (Read)/ (Write)
								//pointer to buffer to supply input data
				&ai,			//3++ Buffer to driver.  Holds port & data (if any).
								//size of input buffer
				sizeof(ai),		//4+++ Length of buffer in bytes.
								//pointer to buffer to receive output data
                &ai,			//5++++ Buffer from driver.
								//size of output buffer
                sizeof(ai),		//6+++++ Length of buffer in bytes. 1/2/4 (char/word/dword)
								//pointer to variable to receive output byte count
				&nOutput,			//7 Bytes placed in DataBufferOut. (store # upon return)
                NULL			//8 NULL means wait till op. completes.
                )				
			)
		{		*pdata = ai.value;		}
	}

	return status;
}


// ======================================================================
// OpenByInterface
//
// Opens nth device found (enumerated) with the given interface class
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
HANDLE CAecMfcUSBDlg::OpenByInterface(GUID *pClassGuid, DWORD instance, PDWORD pError)
{
	HANDLE hDev;
	CDeviceInterfaceClass DevClass(pClassGuid, pError);

	if (*pError != ERROR_SUCCESS)
		return INVALID_HANDLE_VALUE;

	CDeviceInterface DevInterface(&DevClass, instance, pError);

	if (*pError != ERROR_SUCCESS)
		return INVALID_HANDLE_VALUE;

	hDev = CreateFile(
		DevInterface.DevicePath(),
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
		);

	if (hDev == INVALID_HANDLE_VALUE)
		*pError = GetLastError();

	return hDev;
}

*/